/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 * 
 * Copyright 1997-2008 Sun Microsystems, Inc. All rights reserved.
 * Copyright (c) Ericsson AB, 2004-2008. All rights reserved.
 * 
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common Development
 * and Distribution License("CDDL") (collectively, the "License").  You
 * may not use this file except in compliance with the License. You can obtain
 * a copy of the License at https://glassfish.dev.java.net/public/CDDL+GPL.html
 * or glassfish/bootstrap/legal/LICENSE.txt.  See the License for the specific
 * language governing permissions and limitations under the License.
 * 
 * When distributing the software, include this License Header Notice in each
 * file and include the License file at glassfish/bootstrap/legal/LICENSE.txt.
 * Sun designates this particular file as subject to the "Classpath" exception
 * as provided by Sun in the GPL Version 2 section of the License file that
 * accompanied this code.  If applicable, add the following below the License
 * Header, with the fields enclosed by brackets [] replaced by your own
 * identifying information: "Portions Copyrighted [year]
 * [name of copyright owner]"
 * 
 * Contributor(s):
 * 
 * If you wish your version of this file to be governed by only the CDDL or
 * only the GPL Version 2, indicate your decision by adding "[Contributor]
 * elects to include this software in this distribution under the [CDDL or GPL
 * Version 2] license."  If you don't indicate a single choice of license, a
 * recipient has the option to distribute your version of this file under
 * either the CDDL, the GPL Version 2 or to extend the choice of license to
 * its licensees as provided above.  However, if you add GPL Version 2 code
 * and therefore, elected the GPL Version 2 license, then the option applies
 * only if the new code is made subject to such option by the copyright
 * holder.
 */
package org.jvnet.glassfish.comms.clb.core;

import com.sun.enterprise.admin.monitor.callflow.Listener;

import java.util.ArrayList;


/**
 * This class is DCR file event notifier. It listens for admin event related to DCR file and propagates to
 * the listeners waiting for this event.
 *
 * @author kshitiz
 */
public class DCRFileUpdateEventNotifier {
    private ArrayList<DCRFileUpdateEventListener> listeners;

    /** Creates a new instance of DCRFileUpdateEventNotifier */
    public DCRFileUpdateEventNotifier() {
        listeners = new ArrayList<DCRFileUpdateEventListener>();
    }

    /**
     * Add a dcr file update event listener
     *
     * @param listener dcr file update event listener
     */
    public void addDCRFileUpdateEventListener(
        DCRFileUpdateEventListener listener) {
        listeners.add(listener);
    }

    /**
     * This method is called by controller to register this class to admin event registry.
     */
    public void registerWithAdminEventListener() {
        if (listeners.size() > 0) {
            //TBD
            //register in admin event registry to listen for DCR file event
        }
    }

    /**
     * This method propagates dcr file update event to all its listeners.
     */
    public void propagateDCRFileUpdateEvent() {
        for (DCRFileUpdateEventListener listener : listeners) {
            listener.processDCRFileUpdateEvent();
        }
    }
}
