/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 * 
 * Copyright 1997-2008 Sun Microsystems, Inc. All rights reserved.
 * Copyright (c) Ericsson AB, 2004-2008. All rights reserved.
 * 
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common Development
 * and Distribution License("CDDL") (collectively, the "License").  You
 * may not use this file except in compliance with the License. You can obtain
 * a copy of the License at https://glassfish.dev.java.net/public/CDDL+GPL.html
 * or glassfish/bootstrap/legal/LICENSE.txt.  See the License for the specific
 * language governing permissions and limitations under the License.
 * 
 * When distributing the software, include this License Header Notice in each
 * file and include the License file at glassfish/bootstrap/legal/LICENSE.txt.
 * Sun designates this particular file as subject to the "Classpath" exception
 * as provided by Sun in the GPL Version 2 section of the License file that
 * accompanied this code.  If applicable, add the following below the License
 * Header, with the fields enclosed by brackets [] replaced by your own
 * identifying information: "Portions Copyrighted [year]
 * [name of copyright owner]"
 * 
 * Contributor(s):
 * 
 * If you wish your version of this file to be governed by only the CDDL or
 * only the GPL Version 2, indicate your decision by adding "[Contributor]
 * elects to include this software in this distribution under the [CDDL or GPL
 * Version 2] license."  If you don't indicate a single choice of license, a
 * recipient has the option to distribute your version of this file under
 * either the CDDL, the GPL Version 2 or to extend the choice of license to
 * its licensees as provided above.  However, if you add GPL Version 2 code
 * and therefore, elected the GPL Version 2 license, then the option applies
 * only if the new code is made subject to such option by the copyright
 * holder.
 */
package org.jvnet.glassfish.comms.clb.core.common.chr.dcr;

import com.ericsson.ssa.sip.SipServletRequestImpl;
import com.ericsson.ssa.sip.dns.ResolverManager;

import org.jvnet.glassfish.comms.util.LogUtil;

import org.w3c.dom.Element;

import java.util.logging.Level;
import java.util.logging.Logger;


public class SessionCaseCondition extends Operand {
    private static final long serialVersionUID = 1L;
    private static Logger logger = LogUtil.CLB_LOGGER.getLogger();

    @Override
    public void addChild(Element element) {
        super.addChild(element);
    }

    @Override
    public String evaluate(Object object) {
        int conditionType = getConditionType();

        switch (conditionType) {
        case TYPE_EQUALS:

            try {
                SipServletRequestImpl sipreq = (SipServletRequestImpl) object;
                ResolverManager.getInstance().checkRouteHeader(sipreq, false);

                String actualSessionCase = sipreq.getSessionCase().toString();

                if (logger.isLoggable(Level.FINE)) {
                    logger.log(Level.FINE,
                        "SessionCase: " + actualSessionCase);
                }

                String mappingSessionCase = getValue();

                // session-case has to have sub conditions to return true
                if (actualSessionCase.equals(mappingSessionCase)) {
                    return getSubCondition().evaluate(object);
                }
            } catch (Exception e) {
                logger.log(Level.FINE, "" + e);

                return null;
            }

            break;

        case TYPE_CONTAINS:
            return null;

        case TYPE_EXCLUDE:
            return null;
        }

        return null;
    }
}
