/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2008 Sun Microsystems, Inc. All rights reserved.
 * Copyright (c) Ericsson AB, 2004-2008. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common Development
 * and Distribution License("CDDL") (collectively, the "License").  You
 * may not use this file except in compliance with the License. You can obtain
 * a copy of the License at https://glassfish.dev.java.net/public/CDDL+GPL.html
 * or glassfish/bootstrap/legal/LICENSE.txt.  See the License for the specific
 * language governing permissions and limitations under the License.
 *
 * When distributing the software, include this License Header Notice in each
 * file and include the License file at glassfish/bootstrap/legal/LICENSE.txt.
 * Sun designates this particular file as subject to the "Classpath" exception
 * as provided by Sun in the GPL Version 2 section of the License file that
 * accompanied this code.  If applicable, add the following below the License
 * Header, with the fields enclosed by brackets [] replaced by your own
 * identifying information: "Portions Copyrighted [year]
 * [name of copyright owner]"
 *
 * Contributor(s):
 *
 * If you wish your version of this file to be governed by only the CDDL or
 * only the GPL Version 2, indicate your decision by adding "[Contributor]
 * elects to include this software in this distribution under the [CDDL or GPL
 * Version 2] license."  If you don't indicate a single choice of license, a
 * recipient has the option to distribute your version of this file under
 * either the CDDL, the GPL Version 2 or to extend the choice of license to
 * its licensees as provided above.  However, if you add GPL Version 2 code
 * and therefore, elected the GPL Version 2 license, then the option applies
 * only if the new code is made subject to such option by the copyright
 * holder.
 */
package org.jvnet.glassfish.comms.clb.plugin;

import com.sun.enterprise.ee.cms.core.*;

import java.util.Map;
import java.util.Properties;
import java.util.List;
import java.util.ArrayList;
import java.util.logging.Logger;

import org.jvnet.glassfish.comms.util.LogUtil;

/**
 * This class represents the GMS Callback implementation that is registered for listening
 * would be used to do the required initialization
 */

public class GMSCallBackImpl implements CallBack
{
    Properties props = null;
    ClusterConfigEventListener eventListener = null;
    Logger log = LogUtil.CLB_LOGGER.getLogger();
    public GMSCallBackImpl(Properties props)
    {
       this.props = props;
    }

     public void processNotification(Signal signal) {
        log.info("Recieved GMS Notification. Member details = " + signal.getMemberDetails()+
                    "\n signal " + signal +
                    "\n signal.getMemberToken() " + signal.getMemberToken() +
                    "\n signal.getGroupName() " + signal.getGroupName()+
                    "\n signal.getMemberDetails() " + signal.getMemberDetails());
        if (signal instanceof JoinedAndReadyNotificationSignal) {
            addInstance(signal.getGroupName(), signal.getMemberToken(), signal.getMemberDetails());
        } else if (signal instanceof PlannedShutdownSignal || signal instanceof FailureNotificationSignal) {
            removeInstance(signal.getGroupName(), signal.getMemberToken(), signal.getMemberDetails());
        }
    }


    protected void addInstance(String groupName, String memberToken, Map memberDetails)
    {

             ClusterConfigChangeEvent event = new ClusterConfigChangeEvent();
             event.setEventType(ClusterConfigChangeEvent.JOIN_EVENT);
             event.setMemberDetails(memberDetails);
             event.setClusterName(groupName);
             event.setMemberToken(memberToken);
             fireEvent(event);
    }

    protected void removeInstance(String groupName, String memberToken, Map memberDetails)
    {
             ClusterConfigChangeEvent event = new ClusterConfigChangeEvent();
             event.setEventType(ClusterConfigChangeEvent.SHUT_DOWN_EVENT);
             event.setMemberDetails(memberDetails);
             event.setClusterName(groupName);
             event.setMemberToken(memberToken);
             fireEvent(event);
    }
    private List<ClusterConfigEventListener> configEventListeners;

    public void addListener(ClusterConfigEventListener listener)
    {
         if (configEventListeners == null)
         {
             configEventListeners = new ArrayList<ClusterConfigEventListener>();
         }
        configEventListeners.add(listener);
    }

    public void removeListener(ClusterConfigEventListener listener)
    {
         if (configEventListeners != null)
         {
             configEventListeners.remove(listener);
         }
    }

    protected void fireEvent(ClusterConfigChangeEvent event)
    {
          if(configEventListeners != null)
          {
              for (int i =0; i < configEventListeners.size(); i++)
              {
                 ClusterConfigEventListener eventListener =  configEventListeners.get(i);
                 eventListener.handleEvent(event);
              }
          }
    }


}
