/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 * 
 * Copyright 1997-2008 Sun Microsystems, Inc. All rights reserved.
 * Copyright (c) Ericsson AB, 2004-2008. All rights reserved.
 * 
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common Development
 * and Distribution License("CDDL") (collectively, the "License").  You
 * may not use this file except in compliance with the License. You can obtain
 * a copy of the License at https://glassfish.dev.java.net/public/CDDL+GPL.html
 * or glassfish/bootstrap/legal/LICENSE.txt.  See the License for the specific
 * language governing permissions and limitations under the License.
 * 
 * When distributing the software, include this License Header Notice in each
 * file and include the License file at glassfish/bootstrap/legal/LICENSE.txt.
 * Sun designates this particular file as subject to the "Classpath" exception
 * as provided by Sun in the GPL Version 2 section of the License file that
 * accompanied this code.  If applicable, add the following below the License
 * Header, with the fields enclosed by brackets [] replaced by your own
 * identifying information: "Portions Copyrighted [year]
 * [name of copyright owner]"
 * 
 * Contributor(s):
 * 
 * If you wish your version of this file to be governed by only the CDDL or
 * only the GPL Version 2, indicate your decision by adding "[Contributor]
 * elects to include this software in this distribution under the [CDDL or GPL
 * Version 2] license."  If you don't indicate a single choice of license, a
 * recipient has the option to distribute your version of this file under
 * either the CDDL, the GPL Version 2 or to extend the choice of license to
 * its licensees as provided above.  However, if you add GPL Version 2 code
 * and therefore, elected the GPL Version 2 license, then the option applies
 * only if the new code is made subject to such option by the copyright
 * holder.
 */
package org.jvnet.glassfish.comms.gms;


/**
 * This is event listener interface of load-balancer. it handles the cluster membership changes.<br>
 * The cluster membership may change due to following reason<br>
 * <ol>
 * <li>An instance moves to unhealthy state from healthy state</li>
 * <li>An instance moves to healthy state from unhealthy state</li>
 * <li>An instance is enabled</li>
 * <li>An instance is disabled</li>
 * </ol>
 * @author kshitiz
 */
public interface EventListener {
    /**
     * this method handles the event of instance being disabled
     *
     * @param clusterName name of cluster
     * @param instanceName name of instance which is disabled in above mentioned cluster
     */
    public void onDisable(String clusterName, String instanceName);

    /**
     * this method handles the event of instance being enabled
     *
     * @param clusterName name of cluster
     * @param instanceName name of instance which is enabled in above mentioned cluster
     */
    public void onEnable(String clusterName, String instanceName);

    /**
     * this method handles the event of instance becoming healthy
     *
     * @param clusterName name of cluster
     * @param instanceName name of instance which has recovered in above mentioned cluster
     */
    public void onRecovery(String clusterName, String instanceName);

    /**
     * this method handles the event of instance becoming unhealthy
     *
     * @param clusterName name of cluster
     * @param instanceName name of instance which has failed in above mentioned cluster
     */
    public void onFailure(String clusterName, String instanceName);
}
