/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 * 
 * Copyright 1997-2008 Sun Microsystems, Inc. All rights reserved.
 * Copyright (c) Ericsson AB, 2004-2008. All rights reserved.
 * 
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common Development
 * and Distribution License("CDDL") (collectively, the "License").  You
 * may not use this file except in compliance with the License. You can obtain
 * a copy of the License at https://glassfish.dev.java.net/public/CDDL+GPL.html
 * or glassfish/bootstrap/legal/LICENSE.txt.  See the License for the specific
 * language governing permissions and limitations under the License.
 * 
 * When distributing the software, include this License Header Notice in each
 * file and include the License file at glassfish/bootstrap/legal/LICENSE.txt.
 * Sun designates this particular file as subject to the "Classpath" exception
 * as provided by Sun in the GPL Version 2 section of the License file that
 * accompanied this code.  If applicable, add the following below the License
 * Header, with the fields enclosed by brackets [] replaced by your own
 * identifying information: "Portions Copyrighted [year]
 * [name of copyright owner]"
 * 
 * Contributor(s):
 * 
 * If you wish your version of this file to be governed by only the CDDL or
 * only the GPL Version 2, indicate your decision by adding "[Contributor]
 * elects to include this software in this distribution under the [CDDL or GPL
 * Version 2] license."  If you don't indicate a single choice of license, a
 * recipient has the option to distribute your version of this file under
 * either the CDDL, the GPL Version 2 or to extend the choice of license to
 * its licensees as provided above.  However, if you add GPL Version 2 code
 * and therefore, elected the GPL Version 2 license, then the option applies
 * only if the new code is made subject to such option by the copyright
 * holder.
 */
package com.ericsson.ssa.container;

import com.ericsson.ssa.sip.AddressImpl;
import com.ericsson.ssa.sip.Header;
import com.ericsson.ssa.sip.SipServletRequestImpl;
import com.ericsson.ssa.sip.SipServletResponseImpl;
import com.ericsson.ssa.sip.dns.TargetResolver;
import com.ericsson.ssa.sip.dns.TargetTuple;


/**
 *  Implementation  of <code>com.ericsson.ssa.container.SipParserErrorHandler</code> that
 *  writes a terminating response on <code>handleError</code>.
 *
 * @author elnsbur,qmaghes
 * @since Apr 11, 2006
 *
 */
public class SipParserErrorHandlerImpl extends SipParserErrorHandler {
    //Made this explicit for LinkBase now. but could be abstracted to Dispatcher.
    public SipParserErrorHandlerImpl(LinkBase base) {
        super(base);
    }

    public @Override
    void handleError(SipServletRequestImpl req, int code, String errorStr) {
        if (getErrorResponseEnabled()) {
            req.pushTransactionDispatcher(getMyDispatcher());

            SipServletResponseImpl resp = req.createResponseImpl(code, errorStr);

            // Add to tag if there is a to header and there is not yet a to tag 
            if ((resp.getHeader(Header.TO) != null) && (resp.getTo() != null) &&
                    (resp.getTo().getParameter(AddressImpl.TAG_PARAM) == null)) {
                resp.createTag(Header.TO);
            }

            try {
                TargetResolver tr = TargetResolver.getInstance();
                TargetTuple tt = tr.resolveResponse(resp);
                resp.setRemote(tt);
            } catch (Exception e) {
                e.printStackTrace();
                // Fallback if resolve failed, possibly because of Via missing or not parsable.
                resp.setRemote(req.getRemote());
            }

            resp.popDispatcher().dispatch(resp);
        }
    }
}
