/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 * 
 * Copyright 1997-2008 Sun Microsystems, Inc. All rights reserved.
 * Copyright (c) Ericsson AB, 2004-2008. All rights reserved.
 * 
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common Development
 * and Distribution License("CDDL") (collectively, the "License").  You
 * may not use this file except in compliance with the License. You can obtain
 * a copy of the License at https://glassfish.dev.java.net/public/CDDL+GPL.html
 * or glassfish/bootstrap/legal/LICENSE.txt.  See the License for the specific
 * language governing permissions and limitations under the License.
 * 
 * When distributing the software, include this License Header Notice in each
 * file and include the License file at glassfish/bootstrap/legal/LICENSE.txt.
 * Sun designates this particular file as subject to the "Classpath" exception
 * as provided by Sun in the GPL Version 2 section of the License file that
 * accompanied this code.  If applicable, add the following below the License
 * Header, with the fields enclosed by brackets [] replaced by your own
 * identifying information: "Portions Copyrighted [year]
 * [name of copyright owner]"
 * 
 * Contributor(s):
 * 
 * If you wish your version of this file to be governed by only the CDDL or
 * only the GPL Version 2, indicate your decision by adding "[Contributor]
 * elects to include this software in this distribution under the [CDDL or GPL
 * Version 2] license."  If you don't indicate a single choice of license, a
 * recipient has the option to distribute your version of this file under
 * either the CDDL, the GPL Version 2 or to extend the choice of license to
 * its licensees as provided above.  However, if you add GPL Version 2 code
 * and therefore, elected the GPL Version 2 license, then the option applies
 * only if the new code is made subject to such option by the copyright
 * holder.
 */
package com.ericsson.ssa.container.auth;

import org.jvnet.glassfish.comms.deployment.backend.ResourceCollection;
import org.jvnet.glassfish.comms.deployment.backend.SecurityConstraint;
import org.jvnet.glassfish.comms.deployment.backend.SipApplication;
import com.ericsson.ssa.sip.SipServletRequestImpl;
import com.ericsson.ssa.sip.dns.TargetTuple;
import org.jvnet.glassfish.comms.security.auth.impl.DigestAuthenticator;
import org.jvnet.glassfish.comms.security.auth.impl.PAssertedAuthenticator;
import org.jvnet.glassfish.comms.security.authorize.SipResourcePermission;
import org.jvnet.glassfish.comms.security.authorize.SipRoleRefPermission;
import org.jvnet.glassfish.comms.security.authorize.SipUserDataPermission;
import org.jvnet.glassfish.comms.security.util.PolicyBuilder;
import java.net.URL;
import java.security.CodeSource;
import java.security.Policy;
import java.security.Principal;
import java.security.ProtectionDomain;
import java.util.Collection;
import java.util.Collections;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;
import java.util.WeakHashMap;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.security.auth.Subject;
import javax.servlet.sip.SipServletRequest;
import javax.servlet.sip.SipServletResponse;
import org.jvnet.glassfish.comms.security.auth.impl.PAssertedAuthenticatorFactory;
import org.jvnet.glassfish.comms.security.auth.impl.RunAsAuthenticator;

/**
 *
 * @author K.Venugopal@sun.com
 */
public final class AuthModule {

    private SipApplication sipApplicationModel;
    private HashMap<Key, ServletConstraints> servletConstraints = null;
    private PolicyBuilder policyUtil = new PolicyBuilder();
    private String contextId = null;
    private CodeSource codeSource = null;
    private Policy policy = null;
    private String sarName = "";
    private Map protectionDomainCache =
            Collections.synchronizedMap(new WeakHashMap());
    private HashMap<String, String> runAsPrincipals = null;
    private String trustConfigId = "";

    public AuthModule(SipApplication am, String sarName, String contextId) {
        this.contextId = contextId;
        this.runAsPrincipals = new HashMap<String, String>();
        this.sipApplicationModel = am;
        policy = Policy.getPolicy();
        policy.refresh();
        this.sarName = sarName;
        String code = removeSpaces(contextId);

        servletConstraints = new HashMap<Key, ServletConstraints>();

        java.net.URI uri = null;

        try {
            uri = new java.net.URI("file:///" + code);

            if (uri != null) {
                codeSource = new CodeSource(new URL(uri.toString()),
                        (java.security.cert.Certificate[]) null);
            }
        } catch (java.net.URISyntaxException use) {
            //log
            throw new RuntimeException(use);
        } catch (java.net.MalformedURLException mue) {
            //log
            throw new RuntimeException(mue);
        }

        Enumeration<SecurityConstraint> scCollection = sipApplicationModel.getSecurityConstraints();
        //  Iterator<SecurityConstraint> scItr = scCollection.iterator();

        while (scCollection.hasMoreElements()) {
            SecurityConstraint constraint = scCollection.nextElement();
            @SuppressWarnings(value = "unchecked")
            Collection<ResourceCollection> resourceCollection = constraint.getResourceCollections();
            Iterator<ResourceCollection> rcItr = resourceCollection.iterator();

            while (rcItr.hasNext()) {
                ResourceCollection rc = rcItr.next();
                @SuppressWarnings(value = "unchecked")
                Collection<String> sn = rc.getServletNames();
                @SuppressWarnings(value = "unchecked")
                Collection<String> sm = rc.getSipMethods();
                Collection<String> roles = constraint.getAuthorizationConstraintRoleNames();
                Iterator<String> snItr = sn.iterator();

                while (snItr.hasNext()) {
                    String servletName = snItr.next();
                    Iterator<String> smItr = sm.iterator();

                    while (smItr.hasNext()) {
                        String sipMethod = smItr.next();
                        Key key = new Key(servletName, sipMethod);
                        Set<String> roleSet = null;

                        if (servletConstraints.containsKey(key)) {
                            ServletConstraints sc = servletConstraints.get(key);
                            roleSet = sc.roles;
                        } else {
                            roleSet = new HashSet<String>();

                            boolean proxy = constraint.isProxyAuthenticated();
                            ServletConstraints sc = new ServletConstraints(proxy,
                                    roleSet);
                            servletConstraints.put(key, sc);
                        }

                        roleSet.addAll(roles);
                    }
                }
            }
        }
    }

    public final void authenticate(SipServletRequest request, String servletName) {
        if (sipApplicationModel != null) {
            String authMethod =
                    sipApplicationModel.getLoginConfiguration().getAuthenticationMethod();

            if (!((authMethod == null) || (authMethod.length() == 0))) {

                boolean challengeClient = true;
                if ("CANCEL".equals(request.getMethod()) || "ACK".equals(request.getMethod())) {
                    challengeClient = false;
                }
                

                if ("DIGEST".equals(authMethod)) {
                    boolean proxy = isProxy(request.getMethod(),servletName);
                    DigestAuthenticator auth = new DigestAuthenticator(
                            sipApplicationModel.getLoginConfiguration().getRealmName(),proxy);
                    if (!auth.hasAuthorizationHeader(request) && challengeClient) {
                        return;
                    } else {
                        Principal authPrincipal = auth.authenticate(request);
                        if (authPrincipal != null) {
                            ((SipServletRequestImpl) request).setUserPrincipal(authPrincipal);
                            ((SipServletRequestImpl) request).setUser(authPrincipal.getName());
                            removeProcessedAuthHeader(request,proxy);
                        }
                    }
                } else if ("P-Asserted".equals(authMethod)) {
                    PAssertedAuthenticatorFactory paf = PAssertedAuthenticatorFactory.getInstance();
                    //PAssertedAuthenticator auth = new PAssertedAuthenticator(sipApplicationModel.getLoginConfiguration().getRealmName());
                    PAssertedAuthenticator auth = null;
                    if (trustConfigId == null || trustConfigId.length() == 0) {
                        auth = (PAssertedAuthenticator) paf.getAuthenticator();
                    } else {
                        auth = (PAssertedAuthenticator) paf.getAuthenticator(trustConfigId);
                    }

                    TargetTuple remoteHost = ((SipServletRequestImpl) request).getRemote();
                    String host = "";
                    if (remoteHost != null) {
                        host = remoteHost.getIP();
                    }
                    Principal authPrincipal = auth.authenticate(request, host,
                            sipApplicationModel.getLoginConfiguration().getRealmName(), auth.INTERMEDIATE);
                    if (authPrincipal != null) {
                        ((SipServletRequestImpl) request).setUserPrincipal(authPrincipal);
                        ((SipServletRequestImpl) request).setUser(authPrincipal.getName());
                    }
                    return;
                } else {
                    throw new SecurityException(authMethod + " authentication mechanism is not supported");
                }
            }
        }
    }

    private void removeProcessedAuthHeader(SipServletRequest request,boolean isProxy) {
        //TODO: remove the header now, handle this when supporting multiple auth header values
        if (!isProxy) {
            request.removeHeader("Authorization");
            return;
        }
        String pheader = request.getHeader("Proxy-Authorization");        
        request.removeHeader("Proxy-Authorization");
        
    }

    public SipServletResponse createErrorResponse(SipServletRequest request, String servletName) {
        //String authMethod = sipApplicationModel.getLoginConfigAuthMethod();
        String authMethod =
                sipApplicationModel.getLoginConfiguration().getAuthenticationMethod();
        SipServletResponse resp = null;
        int statusCode = 401;
        if ("DIGEST".equals(authMethod)) {
            boolean proxyAuthenticate = false;
            Key key = new Key(servletName, request.getMethod());

            if (servletConstraints.containsKey(key)) {
                ServletConstraints sc = servletConstraints.get(key);
                proxyAuthenticate = sc.proxy;
            }

            if (proxyAuthenticate) {
                statusCode = 407;
            }
            resp = request.createResponse(statusCode);
            DigestAuthenticator auth =
                    new DigestAuthenticator(sipApplicationModel.getLoginConfiguration().getRealmName(),proxyAuthenticate);
            auth.setAuthenticateHeader(resp, request, proxyAuthenticate);
        } else if ("P-Asserted".equals(authMethod)) {
            resp = request.createResponse(403);
        }
        return resp;
    }

    public boolean hasResourcePermissions(SipServletRequest request,
            String servletName) {
        return hasResourcePermissions(request.getMethod(), servletName);
    }

    public boolean hasResourcePermissions(String sipMethod, String servletName) {
        try {

            policyUtil.setPolicyContext(contextId);

            Subject subject = policyUtil.getCurrentSubject();

            Set<Principal> principalSet = subject.getPrincipals();

//            Principal[] principals = (principalSet == null) ? null
//                    : principalSet.toArray(new Principal[0]);

            ProtectionDomain pd =
                    (ProtectionDomain) protectionDomainCache.get(principalSet);
            //ProtectionDomain pd = new ProtectionDomain(codeSource, null, null,principals);

            if (pd == null) {

                Principal[] principals = principals = ((principalSet == null) ? null
                        : (Principal[]) principalSet.toArray(new Principal[0]));
                pd =
                        new ProtectionDomain(codeSource, null, null, principals);
                protectionDomainCache.put(principalSet, pd);

            }

            SipResourcePermission srp = new SipResourcePermission(servletName,
                    sipMethod);

            return policy.implies(pd, srp);
        } catch (Throwable ex) {
            Logger.getLogger(AuthModule.class.getName()).log(Level.SEVERE, null, ex);
            throw new RuntimeException(ex);
        }
    }

    public void setRunAs(Map<String, String> values) {
        this.runAsPrincipals = (HashMap<String, String>) values;
    }

    public Object preSetRunAsIdentity(String servlet) {
        Object currentSC = null;
        if (runAsPrincipals.containsKey(servlet)) {
            String user = runAsPrincipals.get(servlet);
            RunAsAuthenticator ras = new RunAsAuthenticator(user, "");
            ras.authenticate(null);
            currentSC = policyUtil.getSecurityContext();
        }
        return currentSC;
    }

    public void postSetRunAsIdentity(Object oldSC) {
        policyUtil.setSecurityContext(oldSC);
    }

    public boolean checkServletConstraints(SipServletRequest request,
            String servletName) {
        if (hasRoleRefPermission(request, servletName) &&
                hasResourcePermissions(request, servletName)) {
            return true;
        }


        return false;
    }

    /**
     *  All Security constraints must be satisfied.
     */
    public boolean hasRoleRefPermission(SipServletRequest request,
            String servletName) {
        return hasRoleRefPermission(request.getMethod(), servletName);
    }

    public boolean hasRoleRefPermission(String sipMethod, String servletName) {

        try {
            policyUtil.setPolicyContext(contextId);
        } catch (Throwable ex) {
            throw new RuntimeException(ex);
        }

        Subject subject = policyUtil.getCurrentSubject();
        Set<Principal> principalSet = subject.getPrincipals();

        Key key = new Key(servletName, sipMethod);
        ServletConstraints sc = servletConstraints.get(key);

        if (sc == null) {
            return false;
        }

        Set<String> roles = sc.roles;
        Iterator<String> roleItr = roles.iterator();
        boolean isInRole = false;

        ProtectionDomain pd =
                (ProtectionDomain) protectionDomainCache.get(principalSet);
        //ProtectionDomain pd = new ProtectionDomain(codeSource, null, null,principals);

        if (pd == null) {

            Principal[] principals = principals = ((principalSet == null) ? null
                    : (Principal[]) principalSet.toArray(new Principal[0]));
            pd =  new ProtectionDomain(codeSource, null, null, principals);
            protectionDomainCache.put(principalSet, pd);

        }

        while (roleItr.hasNext()) {

            String role = roleItr.next();
            SipRoleRefPermission srp = new SipRoleRefPermission(servletName,
                    role);
            isInRole = policy.implies(pd, srp);
            if (!isInRole) {
                break;
            }

        }

        return isInRole;
    }

    public boolean hasRole(String servletName, String role, Principal principal) {
        try {
            policyUtil.setPolicyContext(contextId);
        } catch (Throwable ex) {
            throw new RuntimeException(ex);
        }
        Subject sub = policyUtil.getSubject(principal);
        Set<Principal> principalSet = sub.getPrincipals();

        ProtectionDomain pd =
                (ProtectionDomain) protectionDomainCache.get(principalSet);

        if (pd == null) {
            Principal[] principals = principals = ((principalSet == null) ? null
                    : (Principal[]) principalSet.toArray(new Principal[0]));
            pd = new ProtectionDomain(codeSource, null, null, principals);
            protectionDomainCache.put(principalSet, pd);
        }
        SipRoleRefPermission srp = new SipRoleRefPermission(servletName,
                role);
        return policy.implies(pd, srp);
    }

    public boolean hasUserDataPermissions(String servletName, String sipMethod,
            boolean isSecured) {

        try {
            policyUtil.setPolicyContext(contextId);
        } catch (Throwable ex) {
            throw new RuntimeException(ex);
        }

        Subject subject = policyUtil.getCurrentSubject();
        Set<Principal> principalSet = subject.getPrincipals();

        boolean isInRole = false;
        Principal[] principals = ((principalSet == null) ? null
                : (Principal[]) principalSet.toArray(new Principal[0]));

         ProtectionDomain pd =
                (ProtectionDomain) protectionDomainCache.get(principalSet);
        if (pd == null) {            
            pd = new ProtectionDomain(codeSource, null, null, principals);
            protectionDomainCache.put(principalSet, pd);
        }
        String transport = "NONE";

        if (isSecured) {
            transport = "CONFIDENTIAL";
        }

        Set<String> sm = new HashSet<String>();
        sm.add(sipMethod);

        SipUserDataPermission wdp = new SipUserDataPermission(servletName, sm,
                transport);

        return policy.implies(pd, wdp);
    }

    private String removeSpaces(String withSpaces) {
        return withSpaces.replace(' ', '_');
    }

    private boolean isProxy(String method, String servletName) {
        Key key = new Key(servletName, method);
        if (servletConstraints.containsKey(key)) {
            ServletConstraints sc = servletConstraints.get(key);
            return sc.proxy;
        }
        return false;
    }
    
    public boolean needsAuthentication(String servletName, String sipMethod) {
        Key key = new Key(servletName, sipMethod);
        if (servletConstraints.containsKey(key)) {
            return true;
        }
        return false;
    }

    class Key {

        String servletName = null;
        String methodName = null;
        int hashCode = -1;

        public Key() {
        }

        public Key(String servletName, String methodName) {
            this.servletName = servletName;
            this.methodName = methodName;
        }

        @Override
        public boolean equals(Object obj) {
            if (!(obj instanceof Key)) {
                return false;
            }

            Key key = (Key) obj;

            if (((servletName != null) && servletName.equals(key.servletName)) &&
                    ((methodName != null) && methodName.equals(key.methodName))) {
                return true;
            }

            return false;
        }

        @Override
        public int hashCode() {
            if (hashCode == -1) {
                StringBuilder builder = new StringBuilder();
                builder.append(servletName);
                builder.append(methodName);
                hashCode = builder.toString().hashCode();
            }

            return hashCode;
        }

        public void reset() {
            servletName = null;
            methodName = null;
        }
    }

    class ServletConstraints {

        boolean proxy = false;
        Set<String> roles = null;

        public ServletConstraints(boolean proxy, Set<String> roles) {
            this.proxy = proxy;
            this.roles = roles;
        }
    }
}
