/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 * 
 * Copyright 1997-2008 Sun Microsystems, Inc. All rights reserved.
 * Copyright (c) Ericsson AB, 2004-2008. All rights reserved.
 * 
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common Development
 * and Distribution License("CDDL") (collectively, the "License").  You
 * may not use this file except in compliance with the License. You can obtain
 * a copy of the License at https://glassfish.dev.java.net/public/CDDL+GPL.html
 * or glassfish/bootstrap/legal/LICENSE.txt.  See the License for the specific
 * language governing permissions and limitations under the License.
 * 
 * When distributing the software, include this License Header Notice in each
 * file and include the License file at glassfish/bootstrap/legal/LICENSE.txt.
 * Sun designates this particular file as subject to the "Classpath" exception
 * as provided by Sun in the GPL Version 2 section of the License file that
 * accompanied this code.  If applicable, add the following below the License
 * Header, with the fields enclosed by brackets [] replaced by your own
 * identifying information: "Portions Copyrighted [year]
 * [name of copyright owner]"
 * 
 * Contributor(s):
 * 
 * If you wish your version of this file to be governed by only the CDDL or
 * only the GPL Version 2, indicate your decision by adding "[Contributor]
 * elects to include this software in this distribution under the [CDDL or GPL
 * Version 2] license."  If you don't indicate a single choice of license, a
 * recipient has the option to distribute your version of this file under
 * either the CDDL, the GPL Version 2 or to extend the choice of license to
 * its licensees as provided above.  However, if you add GPL Version 2 code
 * and therefore, elected the GPL Version 2 license, then the option applies
 * only if the new code is made subject to such option by the copyright
 * holder.
 */

package com.ericsson.ssa.container.callflow;

import java.util.logging.Level;

import org.jvnet.glassfish.comms.util.LogUtil;
import java.util.logging.Logger;

public class CallflowResolver {
    
    private static CallflowResolver _instance = new CallflowResolver();
    
    Logger log = LogUtil.SMI_LOGGER.getLogger();
    
    private String REPORTER_PACKAGE = "org.jvnet.glassfish.comms.admin.callflow.reporter";


    public static CallflowResolver getInstance(){
        return _instance;
    }
    
    public Reporter getReporter(String reporterString){
        if(reporterString != null)
            return getReporter(reporterString.split(","));
        return null;
    }
    
    private Reporter getReporter(String[] reporters){
        if(reporters.length == 1){
            return reporterForString(reporters[0]);
        }
        else {
            ComplexReporter combined = new ComplexReporter();
            for(String name: reporters) {
                Reporter r = reporterForString(name);
                combined.addReporter(r);
            }
            if( combined.countReporters() > 0 )
                return combined;
            return null;
        }
    }
    
    private Reporter reporterForString(String name) {
        String className = REPORTER_PACKAGE + "." + name.trim();
        try {
            Class<?> reporterClass = Class.forName(className);
            Object obj = reporterClass.newInstance();
            if( obj instanceof Reporter ){
                if(log.isLoggable(Level.FINE))
                    log.log(Level.FINE, "Loaded callflow reporter " + className);
                return (Reporter)obj;            
            }
        } catch (InstantiationException ex) {
            log.log(Level.WARNING, "cflow.instantiation_error", new Object[] {className, ex.getMessage()});
        } catch (IllegalAccessException ex) {
            log.log(Level.WARNING, "cflow.instantiation_error", new Object[] {className, ex.getMessage()});
        } catch (ClassNotFoundException ex) {
            log.log(Level.WARNING, "cflow.class_error", new Object[] {className});
        }
        return null;
    }
    
}
