/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 * 
 * Copyright 1997-2008 Sun Microsystems, Inc. All rights reserved.
 * Copyright (c) Ericsson AB, 2004-2008. All rights reserved.
 * 
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common Development
 * and Distribution License("CDDL") (collectively, the "License").  You
 * may not use this file except in compliance with the License. You can obtain
 * a copy of the License at https://glassfish.dev.java.net/public/CDDL+GPL.html
 * or glassfish/bootstrap/legal/LICENSE.txt.  See the License for the specific
 * language governing permissions and limitations under the License.
 * 
 * When distributing the software, include this License Header Notice in each
 * file and include the License file at glassfish/bootstrap/legal/LICENSE.txt.
 * Sun designates this particular file as subject to the "Classpath" exception
 * as provided by Sun in the GPL Version 2 section of the License file that
 * accompanied this code.  If applicable, add the following below the License
 * Header, with the fields enclosed by brackets [] replaced by your own
 * identifying information: "Portions Copyrighted [year]
 * [name of copyright owner]"
 * 
 * Contributor(s):
 * 
 * If you wish your version of this file to be governed by only the CDDL or
 * only the GPL Version 2, indicate your decision by adding "[Contributor]
 * elects to include this software in this distribution under the [CDDL or GPL
 * Version 2] license."  If you don't indicate a single choice of license, a
 * recipient has the option to distribute your version of this file under
 * either the CDDL, the GPL Version 2 or to extend the choice of license to
 * its licensees as provided above.  However, if you add GPL Version 2 code
 * and therefore, elected the GPL Version 2 license, then the option applies
 * only if the new code is made subject to such option by the copyright
 * holder.
 */
package com.ericsson.ssa.dd;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;
import org.jvnet.glassfish.comms.deployment.backend.SessionCase;


/**
 * Operator. The Operator abstract provide common services for the operators:
 * and, or, not
 */
public abstract class Operator implements Condition {
    protected List<Condition> subConditions;

    /**
     * @param condition
     */
    public void addCondition(Condition condition) {
        if (subConditions == null) {
            subConditions = new ArrayList<Condition>();
        }

        subConditions.add(condition);
    }

    /**
     * Get all subconditions of the operator
     *
     * @return The subsconditions of the operator
     */
    public Iterator<Condition> getSubConditions() {
        Iterator<Condition> itr = null;

        if (subConditions == null) {
            itr = Collections.EMPTY_LIST.iterator();
        } else {
            itr = subConditions.iterator();
        }

        return itr;
    }

    /**
     * @param sourceList
     * @param destinationList
     * @throws SipParserException
     */
    protected void addOnlyNewElementToTheList(ArrayList<String> sourceList,
        ArrayList<String> destinationList) throws SipParserException {
        if ((destinationList == null) || (sourceList == null)) {
            throw new SipParserException("AND: impossible Condition: " +
                "addOnlyNewElementToTheList empty List or null");
        }

        if (destinationList.contains(SessionCase.EXTERNAL.toString())) {
            return;
        }

        if (sourceList.contains(SessionCase.EXTERNAL.toString())) {
            destinationList.clear();
            destinationList.add(SessionCase.EXTERNAL.toString());

            return;
        }

        Iterator<String> sourceListIter = sourceList.iterator();
        String element = null;

        while (sourceListIter.hasNext()) {
            element = sourceListIter.next();

            if (!destinationList.contains(element)) {
                destinationList.add(element);
            }
        }
    }
}
